@@ -148,12 +148,13 @@ class CicdStateMgr():
         with open(configFilePath) as f:
             self.configData = yaml.load(f, Loader=yaml.FullLoader)
 
-        with open(secretsFilePath) as f:
-            self.secretData = yaml.load(f, Loader=yaml.FullLoader)
+        if secretsFilePath:
+            with open(secretsFilePath) as f:
+                self.secretData = yaml.load(f, Loader=yaml.FullLoader)
 
         # create all data sources
         dsConfigs = copy.deepcopy(self.configData[CONFIG_DATA_KEY]["datasources"])
-        if 'datasources' in self.secretData[SECRET_DATA_KEY]:
+        if self.secretData and 'datasources' in self.secretData[SECRET_DATA_KEY]:
             dsConfigs = dict_merge(dsConfigs,copy.deepcopy(self.secretData[SECRET_DATA_KEY]["datasources"]))
         self.dataSourceMgr = DataSourceMgr(dsConfigs)
 
@@ -174,7 +175,7 @@ class CicdStateMgr():
             if blankOnError:
                 return ""
             else:
-                return "<jinja2 parse error>"
+                return "<jinja2 parse error> {}".format(origTemplate)
 
     def persist(self, cicdContextData, skipPrimary=False):
         logging.debug("persist() skipPrimary={} cicdContextDataId={}".format(skipPrimary,cicdContextData[STATE][CICD_CONTEXT_DATA_ID]))
@@ -524,10 +525,17 @@ class CicdStateMgr():
 
     def get_via_jsonng_expression(self, getValueExpression, findInData) -> object:
         logging.debug("get_via_jsonng_expression() looking for {}".format(getValueExpression))
-        parsed = jsonpath_ng.parse(getValueExpression)
+
+        try:
+            parsed = jsonpath_ng.parse(getValueExpression)
+        except Exception as e:
+            logging.error("get_via_jsonng_expression() error parsing expression? returning expression as literal value {}. error: {}".format(getValueExpression,str(sys.exc_info()[:2])))
+            return getValueExpression
+
         match = parsed.find(findInData)
 
-        valueToReturn = None
+        # return expression literal by default
+        valueToReturn = getValueExpression
 
         # parsed.find returns an array
         if match and len(match) > 1:
@@ -536,11 +544,14 @@ class CicdStateMgr():
                 toReturn.append(v.value)
 
             valueToReturn = toReturn
-        else:
+        elif match and len(match) == 1:
             valueToReturn = match[0].value
+        else:
+            logging.debug("get_via_jsonng_expression() expression {} yielded nothing, returning literal: {}".format(getValueExpression,getValueExpression))
 
         return valueToReturn
 
+
     def create_template_context(self, cicdContextData:dict, tmplCtxVars:list) -> dict:
         templateContext = copy.deepcopy(cicdContextData)
         templateContext[CONFIG_DATA] = self.configData[CONFIG_DATA_KEY]
@@ -699,14 +710,17 @@ class CicdStateMgr():
             contextDataForTemplate['body'] = responseBodyObj
             
             # always apply global auto capture rules
-            if 'auto-capture-response-data' in self.configData[CONFIG_DATA_KEY]['notify']:
-                captureConfs = self.configData[CONFIG_DATA_KEY]['notify']['auto-capture-response-data']
-                for captureConf in captureConfs:
-                    valueToSet = self.parse_template(str(captureConf['from']),contextDataForTemplate)
-                    if valueToSet and isinstance(valueToSet,str):
-                        valueToSet = valueToSet.strip()
-                        if valueToSet:
-                            self.set_value_in_dict_via_prop_path(cicdContextData, captureConf['to'], valueToSet)
+            if 'notify' in self.configData[CONFIG_DATA_KEY]:
+                if 'auto-capture-response-data' in self.configData[CONFIG_DATA_KEY]['notify']:
+                    captureConfs = self.configData[CONFIG_DATA_KEY]['notify']['auto-capture-response-data']
+                    for captureConf in captureConfs:
+                        valueToSet = self.parse_template(str(captureConf['from']),contextDataForTemplate)
+                        if valueToSet and isinstance(valueToSet,str):
+                            valueToSet = valueToSet.strip()
+                            if valueToSet:
+                                # the 'to' directive can be a template as well
+                                setToKey = self.parse_template(captureConf['to'],contextDataForTemplate)
+                                self.set_value_in_dict_via_prop_path(cicdContextData, setToKey, valueToSet)
 
             # ... and then apply the pipeline specific notify capture rules
             if 'capture-response-data' in notifyConfig:
@@ -716,7 +730,9 @@ class CicdStateMgr():
                     if valueToSet and isinstance(valueToSet,str):
                         valueToSet = valueToSet.strip()
                         if valueToSet:
-                            self.set_value_in_dict_via_prop_path(cicdContextData, captureConf['to'], valueToSet)
+                            # the 'to' directive can be a template as well
+                            setToKey = self.parse_template(captureConf['to'],contextDataForTemplate)
+                            self.set_value_in_dict_via_prop_path(cicdContextData, setToKey, valueToSet)
 
         else:
             logging.error("event_handle_notify() http response code: {} FAILED: {} ".format(response.status_code,response.content))
@@ -751,7 +767,7 @@ class CicdStateMgr():
             'Connection': 'keep-alive'
         }
 
-        logging.debug("event_handle_manual_choice(): POSTing notification to {} : {}".format(notifyUrl,notifyPayload))
+        logging.debug("event_handle_manual_choice(): POSTing manual choices to {} : {}".format(notifyUrl,notifyPayload))
 
         response = requests.request("POST", notifyUrl, data=notifyPayload, headers=headers)
         
